@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2020 Matthias Geier
+# Copyright (c) 2015-2021 Matthias Geier
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,16 +23,18 @@
 https://nbsphinx.readthedocs.io/
 
 """
-__version__ = '0.6.0'
+__version__ = '0.8.4'
 
-import collections
+import collections.abc
 import copy
 import html
 import json
 import os
 import re
 import subprocess
+import sys
 from urllib.parse import unquote
+import uuid
 
 import docutils
 from docutils.parsers import rst
@@ -47,6 +49,13 @@ import sphinx.transforms.post_transforms.images
 from sphinx.util.matching import patmatch
 import traitlets
 
+
+if sys.version_info >= (3, 8) and sys.platform == 'win32':
+    # See: https://github.com/jupyter/jupyter_client/issues/583
+    import asyncio
+
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
 _ipynbversion = 4
 
 logger = sphinx.util.logging.getLogger(__name__)
@@ -96,6 +105,8 @@ RST_TEMPLATE = """
 {% block any_cell %}
 {%- if cell.metadata.nbsphinx != 'hidden' %}
 {{ super() }}
+..
+{# Empty comment to make sure the preceding directive (if any) is closed #}
 {% endif %}
 {%- endblock any_cell %}
 
@@ -154,6 +165,7 @@ RST_TEMPLATE = """
 
     .. image:: {{ output.metadata.filenames[datatype] | posix_path }}
 {%- if datatype in output.metadata %}
+        :class: no-scaled-link
 {%- set width = output.metadata[datatype].width %}
 {%- if width %}
         :width: {{ width }}
@@ -177,7 +189,7 @@ RST_TEMPLATE = """
 
     .. raw:: html
 
-{{ output.data['text/html'] | indent | indent }}
+{{ (output.data['text/html'] or '<!-- empty output -->') | indent | indent }}
 {%- elif datatype == 'application/javascript' %}
 
     .. raw:: html
@@ -192,6 +204,10 @@ RST_TEMPLATE = """
     .. raw:: html
 
         <script type="{{ datatype }}">{{ output.data[datatype] | json_dumps }}</script>
+{%- elif datatype == '' %}
+{# Empty output data #}
+
+    ..
 {% else %}
 
     .. nbwarning:: Data type cannot be displayed: {{ datatype }}
@@ -200,6 +216,8 @@ RST_TEMPLATE = """
 
 
 {% block nboutput -%}
+..
+{# Empty comment to make sure the preceding directive (if any) is closed #}
 {%- set html_datatype, latex_datatype = output | get_output_type %}
 {%- if html_datatype == latex_datatype %}
 {{ insert_nboutput(html_datatype, output, cell) }}
@@ -237,26 +255,22 @@ RST_TEMPLATE = """
 {%- if raw_mimetype == '' %}
 .. raw:: html
 
-{{ cell.source | indent }}
+{{ (cell.source or '<!-- empty raw cell -->') | indent }}
 
 .. raw:: latex
 
-{{ cell.source | indent }}
+{{ (cell.source or '% empty raw cell') | indent }}
 {%- elif raw_mimetype == 'text/html' %}
 .. raw:: html
 
-{{ cell.source | indent }}
+{{ (cell.source or '<!-- empty raw cell -->') | indent }}
 {%- elif raw_mimetype == 'text/latex' %}
 .. raw:: latex
 
-{{ cell.source | indent }}
+{{ (cell.source or '% empty raw cell') | indent }}
 {%- elif raw_mimetype == 'text/markdown' %}
-..
-{# Empty comment to make sure the preceding directive (if any) is closed #}
 {{ cell.source | markdown2rst }}
 {%- elif raw_mimetype == 'text/restructuredtext' %}
-..
-{# Empty comment to make sure the preceding directive (if any) is closed #}
 {{ cell.source }}
 {% endif %}
 {% endblock rawcell %}
@@ -424,23 +438,37 @@ CSS_STRING = """
 /* CSS for nbsphinx extension */
 
 /* remove conflicting styling from Sphinx themes */
-div.nbinput.container,
-div.nbinput.container div.prompt,
-div.nbinput.container div.input_area,
-div.nbinput.container div[class*=highlight],
-div.nbinput.container div[class*=highlight] pre,
-div.nboutput.container,
-div.nboutput.container div.prompt,
-div.nboutput.container div.output_area,
-div.nboutput.container div[class*=highlight],
-div.nboutput.container div[class*=highlight] pre {
-    background: none;
+div.nbinput.container div.prompt *,
+div.nboutput.container div.prompt *,
+div.nbinput.container div.input_area pre,
+div.nboutput.container div.output_area pre,
+div.nbinput.container div.input_area .highlight,
+div.nboutput.container div.output_area .highlight {
     border: none;
-    padding: 0 0;
+    padding: 0;
     margin: 0;
     box-shadow: none;
 }
 
+div.nbinput.container > div[class*=highlight],
+div.nboutput.container > div[class*=highlight] {
+    margin: 0;
+}
+
+div.nbinput.container div.prompt *,
+div.nboutput.container div.prompt * {
+    background: none;
+}
+
+div.nboutput.container div.output_area .highlight,
+div.nboutput.container div.output_area pre {
+    background: unset;
+}
+
+div.nboutput.container div.output_area div.highlight {
+    color: unset;  /* override Pygments text color */
+}
+
 /* avoid gaps between output lines */
 div.nboutput.container div[class*=highlight] pre {
     line-height: normal;
@@ -485,21 +513,34 @@ div.nboutput.container div.prompt pre {
 /* all prompts */
 div.nbinput.container div.prompt,
 div.nboutput.container div.prompt {
-    min-width: %(nbsphinx_prompt_width)s;
-    padding-top: 0.3rem;
-    padding-right: 0.3rem;
-    text-align: right;
-    flex: 0;
+    width: %(nbsphinx_prompt_width)s;
+    padding-top: 5px;
+    position: relative;
+    user-select: none;
 }
+
+div.nbinput.container div.prompt > div,
+div.nboutput.container div.prompt > div {
+    position: absolute;
+    right: 0;
+    margin-right: 0.3ex;
+}
+
 @media (max-width: %(nbsphinx_responsive_width)s) {
     div.nbinput.container div.prompt,
     div.nboutput.container div.prompt {
+        width: unset;
         text-align: left;
         padding: 0.4em;
     }
     div.nboutput.container div.prompt.empty {
         padding: 0;
     }
+
+    div.nbinput.container div.prompt > div,
+    div.nboutput.container div.prompt > div {
+        position: unset;
+    }
 }
 
 /* disable scrollbars on prompts */
@@ -526,7 +567,7 @@ div.nboutput.container div.output_area {
 div.nbinput.container div.input_area {
     border: 1px solid #e0e0e0;
     border-radius: 2px;
-    background: #f5f5f5;
+    /*background: #f5f5f5;*/
 }
 
 /* override MathJax center alignment in output cells */
@@ -591,7 +632,8 @@ div.nboutput.container div.output_area div[class*=highlight].math,
 div.nboutput.container div.output_area.rendered_html,
 div.nboutput.container div.output_area > div.output_javascript,
 div.nboutput.container div.output_area:not(.rendered_html) > img{
-    padding: 0.3rem;
+    padding: 5px;
+    margin: 0;
 }
 
 /* fix copybtn overflow problem in chromium (needed for 'sphinx_copybutton') */
@@ -723,7 +765,7 @@ class Exporter(nbconvert.RSTExporter):
                 # For now, this works only if there is a single MIME bundle
                 (mime_type, data), = bundle.items()
                 text = re.sub(
-                    r'^(\s*\.\. (\|[^|]*\| image|figure)::) attachment:{0}$'
+                    r'^(\s*\.\. ((\|[^|]*\| )?image|figure)::) attachment:{0}$'
                         .format(filename),
                     r'\1 data:{0};base64,{1}'.format(mime_type, data),
                     text, flags=re.MULTILINE)
@@ -812,7 +854,8 @@ class Exporter(nbconvert.RSTExporter):
         def warning(msg, *args):
             logger.warning(
                 '"nbsphinx-thumbnail": ' + msg, *args,
-                location=resources.get('nbsphinx_docname'))
+                location=resources.get('nbsphinx_docname'),
+                type='nbsphinx', subtype='thumbnail')
             thumbnail['filename'] = _BROKEN_THUMBNAIL
 
         for cell_index, cell in enumerate(nb.cells):
@@ -895,6 +938,7 @@ class NotebookParser(rst.Parser):
             CreateNotebookSectionAnchors,
             ReplaceAlertDivs,
             CopyLinkedFiles,
+            ForceEquations,
         ]
 
     def parse(self, inputstring, document):
@@ -928,7 +972,8 @@ class NotebookParser(rst.Parser):
         else:
             raise NotebookError(
                 'No converter was found for {!r}'.format(srcfile))
-        if isinstance(converter, collections.Sequence):
+        if (isinstance(converter, collections.abc.Sequence) and
+                not isinstance(converter, str)):
             if len(converter) != 2:
                 raise NotebookError(
                     'The values of nbsphinx_custom_formats must be '
@@ -1005,6 +1050,7 @@ class NotebookParser(rst.Parser):
 
         if resources.get('nbsphinx_orphan', False):
             rst.Parser.parse(self, ':orphan:', document)
+        rst.Parser.parse(self, '.. highlight:: none', document)
         if env.config.nbsphinx_prolog:
             prolog = exporter.environment.from_string(
                 env.config.nbsphinx_prolog).render(env=env)
@@ -1222,6 +1268,9 @@ class CitationParser(html.parser.HTMLParser):
             if name == 'data-cite':
                 self.cite = ':cite:`' + value + '`'
                 return True
+            elif name == 'data-footcite':
+                self.cite = ':footcite:`' + value + '`'
+                return True
         return False
 
     def reset(self):
@@ -1231,6 +1280,53 @@ class CitationParser(html.parser.HTMLParser):
         self.cite = ''
 
 
+class ImgParser(html.parser.HTMLParser):
+    """Turn HTML <img> tags into raw RST blocks."""
+
+    def handle_starttag(self, tag, attrs):
+        self._check_img(tag, attrs)
+
+    def handle_startendtag(self, tag, attrs):
+        self._check_img(tag, attrs)
+
+    def _check_img(self, tag, attrs):
+        if tag != 'img':
+            return
+        # NB: attrs is a list of pairs
+        attrs = dict(attrs)
+        if 'src' not in attrs:
+            return
+        img_path = nbconvert.filters.posix_path(attrs['src'])
+        if img_path.startswith('data'):
+            # Allow multi-line data, see issue #474
+            img_path = img_path.replace('\n', '')
+        lines = ['image:: ' + img_path]
+        indent = ' ' * 4
+        classes = []
+        if 'class' in attrs:
+            classes.append(attrs['class'])
+        if 'alt' in attrs:
+            lines.append(indent + ':alt: ' + attrs['alt'])
+        if 'width' in attrs:
+            lines.append(indent + ':width: ' + attrs['width'])
+        if 'height' in attrs:
+            lines.append(indent + ':height: ' + attrs['height'])
+        if {'width', 'height'}.intersection(attrs):
+            classes.append('no-scaled-link')
+        if classes:
+            lines.append(indent + ':class: ' + ' '.join(classes))
+
+        definition = '\n'.join(lines)
+        hex_id = uuid.uuid4().hex
+        definition = '.. |' + hex_id + '| ' + definition
+        self.obj = {'t': 'RawInline', 'c': ['rst', '|' + hex_id + '|']}
+        self.definition = definition
+
+    def reset(self):
+        super().reset()
+        self.obj = {}
+
+
 def markdown2rst(text):
     """Convert a Markdown string to reST via pandoc.
 
@@ -1245,16 +1341,22 @@ def markdown2rst(text):
 
     """
 
-    def parse_html(obj):
+    def parse_citation(obj):
         p = CitationParser()
         p.feed(obj['c'][1])
         p.close()
         return p
 
+    def parse_img(obj):
+        p = ImgParser()
+        p.feed(obj['c'][1])
+        p.close()
+        return p
+
     def object_hook(obj):
         if object_hook.open_cite_tag:
             if obj.get('t') == 'RawInline' and obj['c'][0] == 'html':
-                p = parse_html(obj)
+                p = parse_citation(obj)
                 if p.endtag == object_hook.open_cite_tag:
                     object_hook.open_cite_tag = ''
             return {'t': 'Str', 'c': ''}  # Object is replaced by empty string
@@ -1273,14 +1375,20 @@ def markdown2rst(text):
             obj = {'t': 'RawInline',
                    'c': ['rst', ':nbsphinx-math:`{}`'.format(obj['c'][1])]}
         elif obj.get('t') == 'RawInline' and obj['c'][0] == 'html':
-            p = parse_html(obj)
+            p = parse_citation(obj)
             if p.starttag:
                 object_hook.open_cite_tag = p.starttag
             if p.cite:
                 obj = {'t': 'RawInline', 'c': ['rst', p.cite]}
+            if not p.starttag and not p.cite:
+                p = parse_img(obj)
+                if p.obj:
+                    obj = p.obj
+                    object_hook.image_definitions.append(p.definition)
         return obj
 
     object_hook.open_cite_tag = ''
+    object_hook.image_definitions = []
 
     def filter_func(text):
         json_data = json.loads(text, object_hook=object_hook)
@@ -1293,10 +1401,14 @@ def markdown2rst(text):
         input_format += '-native_divs+raw_html'
 
     rststring = pandoc(text, input_format, 'rst', filter_func=filter_func)
-    return re.sub(r'^\n( *)\x0e:nowrap:\x0f$',
-                  r'\1:nowrap:',
-                  rststring,
-                  flags=re.MULTILINE)
+    rststring = re.sub(
+        r'^\n( *)\x0e:nowrap:\x0f$',
+        r'\1:nowrap:',
+        rststring,
+        flags=re.MULTILINE)
+    rststring += '\n\n'
+    rststring += '\n'.join(object_hook.image_definitions)
+    return rststring
 
 
 def pandoc(source, fmt, to, filter_func=None):
@@ -1539,7 +1651,8 @@ class CreateNotebookSectionAnchors(docutils.transforms.Transform):
         if not all_ids:
             logger.warning(
                 'Each notebook should have at least one section title',
-                location=self.document[0])
+                location=self.document[0],
+                type='nbsphinx', subtype='notebooktitle')
 
 
 class CreateSectionLabels(docutils.transforms.Transform):
@@ -1672,15 +1785,33 @@ class CopyLinkedFiles(docutils.transforms.Transform):
             file = os.path.normpath(
                 os.path.join(os.path.dirname(env.docname), relpath))
             if not os.path.isfile(os.path.join(env.srcdir, file)):
-                logger.warning('File not found: %r', file, location=node)
+                logger.warning(
+                    'File not found: %r', file, location=node,
+                    type='nbsphinx', subtype='localfile')
                 continue  # Link is ignored
             elif file.startswith('..'):
-                logger.warning('Link outside source directory: %r', file,
-                               location=node)
+                logger.warning(
+                    'Link outside source directory: %r', file, location=node,
+                    type='nbsphinx', subtype='localfile')
                 continue  # Link is ignored
             env.nbsphinx_files.setdefault(env.docname, []).append(file)
 
 
+class ForceEquations(docutils.transforms.Transform):
+    """Unconditionally enable equations on notebooks.
+
+    Except if ``nbsphinx_assume_equations`` is set to ``False``.
+
+    """
+
+    default_priority = 900  # after checking for equations in MathDomain
+
+    def apply(self):
+        env = self.document.settings.env
+        if env.config.nbsphinx_assume_equations:
+            env.get_domain('math').data['has_equations'][env.docname] = True
+
+
 class GetSizeFromImages(
         sphinx.transforms.post_transforms.images.BaseImageConverter):
     """Get size from images and store it as node attributes.
@@ -1732,50 +1863,55 @@ def patched_toctree_resolve(self, docname, builder, toctree, *args, **kwargs):
 
 
 def config_inited(app, config):
-    # Set default value for CSS prompt width (optimized for two-digit numbers)
-    if config.nbsphinx_prompt_width is None:
-        config.nbsphinx_prompt_width = {
-            'agogo': '4.5ex',
-            'alabaster': '5.5ex',
-            'alabaster_jupyterhub': '5.5ex',
-            'basicstrap': '5.5ex',
-            'better': '4.5ex',
-            'bizstyle': '5.5ex',
-            'bootstrap': '5.5ex',
-            'bootstrap-astropy': '5.5ex',
-            'classic': '4.5ex',
-            'cloud': '5ex',
-            'dotted': '5ex',
-            'greencloud': '5ex',
-            'guzzle_sphinx_theme': '5.5ex',
-            'haiku': '4.5ex',
-            'julia': '5.5ex',
-            'jupyter': '5.5ex',
-            'maisie_sphinx_theme': '5.5ex',
-            'nature': '5ex',
-            'pandas_sphinx_theme': '5.5ex',
-            'pangeo': '5ex',
-            'pyramid': '4.5ex',
-            'pytorch_sphinx_theme': '14ex',
-            'redcloud': '5ex',
-            'scrolls': '5.5ex',
-            'sizzle': '5ex',
-            'sphinxdoc': '5.5ex',
-            'sphinx_material': '5.5ex',
-            'sphinx_py3doc_enhanced_theme': '6.5ex',
-            'sphinx_pyviz_theme': '5.5ex',
-            'sphinx_rtd_theme': '5ex',
-            'sphinx_typlog_theme': '5.5ex',
-            'traditional': '5ex',
-        }.get(config.html_theme, '7ex')
-
     for suffix in config.nbsphinx_custom_formats:
         app.add_source_suffix(suffix, 'jupyter_notebook')
 
+    if '**.ipynb_checkpoints' not in config.exclude_patterns:
+        config.exclude_patterns.append('**.ipynb_checkpoints')
+
     # Make sure require.js is loaded after all other extensions,
     # see https://github.com/spatialaudio/nbsphinx/issues/409
     app.connect('builder-inited', load_requirejs)
 
+    # See also https://github.com/sphinx-doc/sphinx/pull/5504
+    if hasattr(config, 'mathjax3_config') and config.mathjax2_config is None:
+        # NB: If mathjax_path is used in Sphinx >= 4 to load MathJax v2,
+        # this only works if mathjax_config or mathjax2_config is specified.
+        if config.mathjax3_config is None:
+            config.mathjax3_config = {}
+        mathjax3_config = config.mathjax3_config
+        tex = {
+            'inlineMath': [['$', '$'], ['\\(', '\\)']],
+            'processEscapes': True,
+        }
+        tex.update(mathjax3_config.get('tex', {}))
+        mathjax3_config['tex'] = tex
+        options = {
+            'ignoreHtmlClass': 'document',
+            'processHtmlClass': 'math|output_area',
+        }
+        options.update(mathjax3_config.get('options', {}))
+        mathjax3_config['options'] = options
+    else:
+        if hasattr(config, 'mathjax2_config'):
+            # Sphinx >= 4.0
+            if config.mathjax2_config is None:
+                config.mathjax2_config = {}
+            mathjax2_config = config.mathjax2_config
+        else:
+            # Sphinx < 4.0
+            if config.mathjax_config is None:
+                config.mathjax_config = {}
+            mathjax2_config = config.mathjax_config
+        tex2jax = {
+            'inlineMath': [['$', '$'], ['\\(', '\\)']],
+            'processEscapes': True,
+            'ignoreClass': 'document',
+            'processClass': 'math|output_area',
+        }
+        tex2jax.update(mathjax2_config.get('tex2jax', {}))
+        mathjax2_config['tex2jax'] = tex2jax
+
 
 def load_requirejs(app):
     config = app.config
@@ -1795,6 +1931,7 @@ def load_requirejs(app):
 
 def builder_inited(app):
     env = app.env
+    env.settings['line_length_limit'] = 100_000_000
     env.nbsphinx_notebooks = {}
     env.nbsphinx_files = {}
     if not hasattr(env, 'nbsphinx_thumbnails'):
@@ -1804,6 +1941,30 @@ def builder_inited(app):
     sphinx.util.ensuredir(env.nbsphinx_auxdir)
 
 
+def env_merge_info(app, env, docnames, other):
+    env.nbsphinx_notebooks.update(other.nbsphinx_notebooks)
+    env.nbsphinx_files.update(other.nbsphinx_files)
+    env.nbsphinx_thumbnails.update(other.nbsphinx_thumbnails)
+    env.nbsphinx_widgets.update(other.nbsphinx_widgets)
+
+
+def env_purge_doc(app, env, docname):
+    """Remove list of local files for a given document."""
+    try:
+        del env.nbsphinx_notebooks[docname]
+    except KeyError:
+        pass
+    try:
+        del env.nbsphinx_files[docname]
+    except KeyError:
+        pass
+    try:
+        del env.nbsphinx_thumbnails[docname]
+    except KeyError:
+        pass
+    env.nbsphinx_widgets.discard(docname)
+
+
 def html_page_context(app, pagename, templatename, context, doctree):
     """Add CSS string to HTML pages that contain code cells."""
     style = ''
@@ -1834,7 +1995,9 @@ def html_collect_pages(app):
         try:
             sphinx.util.copyfile(os.path.join(app.env.srcdir, file), target)
         except OSError as err:
-            logger.warning('Cannot copy local file %r: %s', file, err)
+            logger.warning(
+                'Cannot copy local file %r: %s', file, err,
+                type='nbsphinx', subtype='localfile')
     notebooks = app.env.nbsphinx_notebooks.values()
     for notebook in status_iterator(
             notebooks, 'copying notebooks ... ',
@@ -1845,23 +2008,6 @@ def html_collect_pages(app):
     return []  # No new HTML pages are created
 
 
-def env_purge_doc(app, env, docname):
-    """Remove list of local files for a given document."""
-    try:
-        del env.nbsphinx_notebooks[docname]
-    except KeyError:
-        pass
-    try:
-        del env.nbsphinx_files[docname]
-    except KeyError:
-        pass
-    try:
-        del env.nbsphinx_thumbnails[docname]
-    except KeyError:
-        pass
-    env.nbsphinx_widgets.discard(docname)
-
-
 def env_updated(app, env):
     widgets_path = app.config.nbsphinx_widgets_path
     if widgets_path is None:
@@ -1871,7 +2017,8 @@ def env_updated(app, env):
             except ImportError:
                 logger.warning(
                     'nbsphinx_widgets_path not given '
-                    'and ipywidgets module unavailable')
+                    'and ipywidgets module unavailable',
+                    type='nbsphinx', subtype='ipywidgets')
             else:
                 widgets_path = DEFAULT_EMBED_REQUIREJS_URL
         else:
@@ -1918,7 +2065,8 @@ def doctree_resolved(app, doctree, fromdocname):
                                     logger.warning(
                                         'page %s matches two patterns in '
                                         'nbsphinx_thumbnails: %r and %r',
-                                        doc, matched, pattern)
+                                        doc, matched, pattern,
+                                        type='nbsphinx', subtype='thumbnail')
                                 # else the already matched pattern is more
                                 # specific than the present one, because it
                                 # contains no wildcard
@@ -1944,7 +2092,7 @@ def doctree_resolved(app, doctree, fromdocname):
             else:
                 logger.warning(
                     'External links are not supported in gallery: %s', doc,
-                    location=fromdocname)
+                    location=fromdocname, type='nbsphinx', subtype='gallery')
         gallery = GalleryNode()
         gallery['entries'] = entries
         toctree['nbsphinx_gallery'] = True
@@ -2110,8 +2258,7 @@ def setup(app):
     app.add_config_value('nbsphinx_allow_errors', False, rebuild='')
     app.add_config_value('nbsphinx_timeout', None, rebuild='')
     app.add_config_value('nbsphinx_codecell_lexer', 'none', rebuild='env')
-    # Default value is set in config_inited():
-    app.add_config_value('nbsphinx_prompt_width', None, rebuild='html')
+    app.add_config_value('nbsphinx_prompt_width', '4.5ex', rebuild='html')
     app.add_config_value('nbsphinx_responsive_width', '540px', rebuild='html')
     app.add_config_value('nbsphinx_prolog', None, rebuild='env')
     app.add_config_value('nbsphinx_epilog', None, rebuild='env')
@@ -2126,6 +2273,7 @@ def setup(app):
     app.add_config_value('nbsphinx_widgets_path', None, rebuild='html')
     app.add_config_value('nbsphinx_widgets_options', {}, rebuild='html')
     app.add_config_value('nbsphinx_thumbnails', {}, rebuild='html')
+    app.add_config_value('nbsphinx_assume_equations', True, rebuild='env')
 
     app.add_directive('nbinput', NbInput)
     app.add_directive('nboutput', NbOutput)
@@ -2151,6 +2299,7 @@ def setup(app):
     app.connect('env-purge-doc', env_purge_doc)
     app.connect('env-updated', env_updated)
     app.connect('doctree-resolved', doctree_resolved)
+    app.connect('env-merge-info', env_merge_info)
     app.add_transform(CreateSectionLabels)
     app.add_transform(CreateDomainObjectLabels)
     app.add_transform(RewriteLocalLinks)
@@ -2161,18 +2310,6 @@ def setup(app):
     # see https://github.com/sphinx-doc/sphinx/issues/2155:
     rst.directives.register_directive('code', sphinx.directives.code.CodeBlock)
 
-    # Work-around until https://github.com/sphinx-doc/sphinx/pull/5504 is done:
-    mathjax_config = app.config._raw_config.setdefault('mathjax_config', {})
-    mathjax_config.setdefault(
-        'tex2jax',
-        {
-            'inlineMath': [['$', '$'], ['\\(', '\\)']],
-            'processEscapes': True,
-            'ignoreClass': 'document',
-            'processClass': 'math|output_area',
-        }
-    )
-
     # Add LaTeX definitions to preamble
     latex_elements = app.config._raw_config.setdefault('latex_elements', {})
     latex_elements['preamble'] = '\n'.join([
